# -*- coding: utf-8 -*-
"""Global program initialization."""
# SPDX-FileCopyrightText: 2018-2023 Sven Sager
# SPDX-License-Identifier: LGPL-2.0-or-later
__author__ = "Sven Sager"
__copyright__ = "Copyright (C) 2018-2023 Sven Sager"
__license__ = "LGPL-2.0-or-later"
__version__ = "1.3.1"

import logging
import sys
from argparse import ArgumentParser
from configparser import ConfigParser
from os import R_OK, W_OK, access, environ, getpid, remove
from os.path import abspath, dirname, exists, join
from shutil import copy, move
from threading import Event

try:
    # Import program version from meta data module of your program
    from . import __version__ as external_version
except Exception:
    external_version = None

# Program name
programname = "revpicommander"
program_version = external_version

conf_rw = False  # If you want so save the configuration with .save_conf() set to True
conf_rw_save = False  # Create new conf file in same directory and move to old one
conf_rw_backup = False  # Keep a backup of old conf file [filename].bak
_extend_daemon_startup_timeout = 0.0  # Default startup timeout is 90 seconds

conf = ConfigParser()
logger = logging.getLogger()
pidfile = "/var/run/{0}.pid".format(programname)
_daemon_started_up = Event()
_daemon_main_pid = getpid()
_systemd_notify = environ.get("NOTIFY_SOCKET", None)
if _systemd_notify:
    from socket import AF_UNIX, SOCK_DGRAM, socket

    # Set up the notification socket for systemd communication
    _systemd_socket = socket(family=AF_UNIX, type=SOCK_DGRAM)
    if _extend_daemon_startup_timeout:
        # Extend systemd TimeoutStartSec by defined timeout extension in micro seconds
        _systemd_socket.sendto(
            f"EXTEND_TIMEOUT_USEC={_extend_daemon_startup_timeout * 1000000}\n",
            _systemd_notify,
        )


def can_be_forked():
    """
    Check the possibility of forking the process.

    Under certain circumstances, a process cannot be forked. These include
    certain build settings or packaging, as well as the missing function on
    some operating systems.

    :return: True, if forking is possible
    """
    from sys import platform

    # Windows operating system does not support the .fork() call
    if platform.startswith("win"):
        return False

    # A PyInstaller bundle does not support the .fork() call
    if getattr(sys, "frozen", False) and hasattr(sys, "_MEIPASS"):
        return False

    return True


def cleanup():
    """
    Clean up before exit the program.

    This function must be called at the end of the program. It flushes
    the logging buffers and deletes the PID file in daemon mode.
    """
    if pargs.daemon and exists(pidfile):
        remove(pidfile)

    # Shutdown logging system
    logging.shutdown()

    # Close logfile
    if pargs.daemon:
        sys.stdout.close()


def reconfigure_logger():
    """Configure logging module of program."""

    class FilterDebug(logging.Filter):
        """Set this filter to log handler if verbose level is > 1."""

        def filter(self, record: logging.LogRecord) -> bool:
            remove_record = False

            # Remove paramiko ssh module
            remove_record = remove_record or record.name.startswith("paramiko")

            return not remove_record

    # Clear all log handler
    for lhandler in logger.handlers.copy():
        lhandler.close()
        logger.removeHandler(lhandler)

    if pargs.daemon:
        # Create daemon log file
        fh_logfile = open("/var/log/{0}.log".format(programname), "a")

        # Close stdout and use logfile
        sys.stdout.close()
        sys.stdout = fh_logfile
        sys.stderr = sys.stdout

    # Create new log handler
    if pargs.verbose > 2:
        log_frm = "{asctime} [{levelname:8}] {name} {message}"
    else:
        log_frm = "{asctime} [{levelname:8}] {message}"
    logformat = logging.Formatter(log_frm, datefmt="%Y-%m-%d %H:%M:%S", style="{")
    lhandler = logging.StreamHandler(sys.stdout)
    lhandler.setFormatter(logformat)
    logger.addHandler(lhandler)

    if "logfile" in pargs and pargs.logfile is not None:
        # Write logs to a logfile
        lhandler = logging.FileHandler(filename=pargs.logfile)
        lhandler.setFormatter(logformat)
        logger.addHandler(lhandler)

    # Loglevel auswerten
    if pargs.verbose == 1:
        loglevel = logging.INFO
    elif pargs.verbose > 1:
        lhandler.addFilter(FilterDebug())
        loglevel = logging.DEBUG
    else:
        loglevel = logging.WARNING
    logger.setLevel(loglevel)


def reload_conf(clear_load=False) -> None:
    """
    Reload config file.

    After successful reload, call set_startup_complete() function to inform
    systemd that all functions are available again.

    If keys are commented out in conf file, they will still be in the conf file.
    To remove not existing keys set clear_load to True.

    :param clear_load: Clear conf before reload
    """
    if _systemd_notify:
        # Inform systemd about reloading configuration
        _systemd_socket.sendto(b"RELOADING=1\n", _systemd_notify)

        # Reset started up event for the set_startup_complete function
        _daemon_started_up.clear()

    if "conffile" in pargs:
        # Check config file
        if not access(pargs.conffile, R_OK):
            raise RuntimeError("can not access config file '{0}'".format(pargs.conffile))
        if conf_rw:
            if (conf_rw_save or conf_rw_backup) and not access(dirname(pargs.conffile), W_OK):
                raise RuntimeError(
                    "can not wirte to directory '{0}' to create files"
                    "".format(dirname(pargs.conffile))
                )
            if not access(pargs.conffile, W_OK):
                raise RuntimeError("can not write to config file '{0}'".format(pargs.conffile))

        if clear_load:
            # Clear all sections and do not create a new instance
            for section in conf.sections():
                conf.remove_section(section)

        # Read configuration
        logger.info("loading config file: {0}".format(pargs.conffile))
        conf.read(pargs.conffile)


def save_conf():
    """Save configuration."""
    if not conf_rw:
        raise RuntimeError("You have to set conf_rw to True.")
    if "conffile" in pargs:
        if conf_rw_backup:
            copy(pargs.conffile, pargs.conffile + ".bak")
        if conf_rw_save:
            with open(pargs.conffile + ".new", "w") as fh:
                conf.write(fh)
            move(pargs.conffile + ".new", pargs.conffile)
        else:
            with open(pargs.conffile, "w") as fh:
                conf.write(fh)


def startup_complete():
    """
    Call this when the daemon is completely started.

    When a daemon is started, it may take some time for everything to be
    available. This function notifies the init system when all functions of
    this daemon are available so that the starts of further daemons can be
    properly timed.

    The systemd unit file that is supposed to start this demon must be set
    to 'Type=notify'. If the daemon supports reloading the settings,
    'ExecReload=/bin/kill -HUP $MAINPID' must also be set. The daemon must
    call this function again after the reload in order to signal systemd the
    completed reload.

    If systemd is available from version 250 and the daemon supports reloading
    the settings, 'Type=notify-reload' can be used without 'ExecReload'. The
    type 'notify-reload' is preferable if possible, as the reloading of the
    daemon is also synchronized with systemd.

    If the '--fork' parameter is used, the main process ends after calling
    this function to prevent the further start of demons by other init systems.
    """
    if _daemon_started_up.is_set():
        # Everyone was notified about complete start, if set
        return

    if _systemd_notify:
        # Inform systemd about complete startup of daemon process
        _systemd_socket.sendto(b"READY=1\n", _systemd_notify)

    if pargs.daemon:
        from os import kill

        # Send SIGTERM signal to main process
        kill(_daemon_main_pid, 15)

    _daemon_started_up.set()


# Generate command arguments of the program
parser = ArgumentParser(
    prog=programname,
    # todo: Add program description for help
    description="Program description",
)
parser.add_argument("--version", action="version", version=f"%(prog)s {program_version}")
parser.add_argument(
    "-f",
    "--logfile",
    dest="logfile",
    help="save log entries to this file",
)
parser.add_argument(
    "-v",
    "--verbose",
    action="count",
    dest="verbose",
    default=0,
    help="switch on verbose logging",
)
# If packed with opensource licenses, add argument to print license information about bundled modules
open_source_licenses = join(dirname(__file__), "open-source-licenses", "open-source-licenses.txt")
if exists(open_source_licenses):
    parser.add_argument(
        "--open-source-licenses",
        action="store_true",
        dest="oss_licenses",
        help="print packed open-source-licenses and exit",
    )
pargs = parser.parse_args()

# Process open-source-licenses argument, if set (only affects bundled apps)
if "oss_licenses" in pargs and pargs.oss_licenses:
    with open(open_source_licenses, "r") as fh:
        sys.stdout.write(fh.read())
    sys.exit(0)

# Check important objects and set to default if they do not exists
if "daemon" not in pargs:
    pargs.daemon = False
if "verbose" not in pargs:
    pargs.verbose = 0

# Check if the program should run as a daemon
if pargs.daemon:
    # Check if daemon is already running
    if exists(pidfile):
        logger.error("Program already running as daemon. Check '{0}'".format(pidfile))
        sys.exit(1)

    # Fork to daemon
    from os import fork

    pid = fork()
    if pid > 0:
        # Main process waits for exit till startup is complete
        from os import kill
        from signal import SIGKILL, SIGTERM, signal

        # Catch the TERM signal, which will be sent from the forked process after startup_complete
        signal(SIGTERM, lambda number, frame: _daemon_started_up.set())

        # Use the default timeout of 90 seconds from systemd also for the '--daemon' flag
        if not _daemon_started_up.wait(90.0 + _extend_daemon_startup_timeout):
            sys.stderr.write(
                "Run into startup complete timout! Killing fork and exit main process\n"
            )
            kill(pid, SIGKILL)
            sys.exit(1)

        # Main process writes pidfile with pid of forked process
        with open(pidfile, "w") as f:
            f.write(str(pid))

        sys.exit(0)

# Get absolute paths
pwd = abspath(".")

# Configure logger
if "logfile" in pargs and pargs.logfile is not None and dirname(pargs.logfile) == "":
    pargs.logfile = join(pwd, pargs.logfile)
reconfigure_logger()

# Initialize configparser of globalconfig
if "conffile" in pargs and dirname(pargs.conffile) == "":
    pargs.conffile = join(pwd, pargs.conffile)

# Load configuration - Comment out, if you do that in your own program
reload_conf()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'oss_licenses.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_diag_oss_licenses(object):
    def setupUi(self, diag_oss_licenses):
        diag_oss_licenses.setObjectName("diag_oss_licenses")
        diag_oss_licenses.resize(640, 480)
        self.verticalLayout = QtWidgets.QVBoxLayout(diag_oss_licenses)
        self.verticalLayout.setObjectName("verticalLayout")
        self.splitter = QtWidgets.QSplitter(diag_oss_licenses)
        self.splitter.setOrientation(QtCore.Qt.Vertical)
        self.splitter.setChildrenCollapsible(False)
        self.splitter.setObjectName("splitter")
        self.tb_oss_licenses = QtWidgets.QTableWidget(self.splitter)
        self.tb_oss_licenses.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tb_oss_licenses.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tb_oss_licenses.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tb_oss_licenses.setCornerButtonEnabled(False)
        self.tb_oss_licenses.setObjectName("tb_oss_licenses")
        self.tb_oss_licenses.setColumnCount(2)
        self.tb_oss_licenses.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.tb_oss_licenses.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tb_oss_licenses.setHorizontalHeaderItem(1, item)
        self.tb_oss_licenses.horizontalHeader().setSortIndicatorShown(True)
        self.tb_oss_licenses.horizontalHeader().setStretchLastSection(True)
        self.tb_oss_licenses.verticalHeader().setVisible(False)
        self.txt_license = QtWidgets.QTextEdit(self.splitter)
        self.txt_license.setTabChangesFocus(True)
        self.txt_license.setUndoRedoEnabled(False)
        self.txt_license.setLineWrapMode(QtWidgets.QTextEdit.NoWrap)
        self.txt_license.setReadOnly(True)
        self.txt_license.setObjectName("txt_license")
        self.verticalLayout.addWidget(self.splitter)
        self.btn_box = QtWidgets.QDialogButtonBox(diag_oss_licenses)
        self.btn_box.setOrientation(QtCore.Qt.Horizontal)
        self.btn_box.setStandardButtons(QtWidgets.QDialogButtonBox.Close)
        self.btn_box.setObjectName("btn_box")
        self.verticalLayout.addWidget(self.btn_box)
        self.action_start = QtWidgets.QAction(diag_oss_licenses)
        self.action_start.setObjectName("action_start")

        self.retranslateUi(diag_oss_licenses)
        self.btn_box.accepted.connect(diag_oss_licenses.accept) # type: ignore
        self.btn_box.rejected.connect(diag_oss_licenses.reject) # type: ignore
        self.action_start.triggered.connect(diag_oss_licenses.exec) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(diag_oss_licenses)

    def retranslateUi(self, diag_oss_licenses):
        _translate = QtCore.QCoreApplication.translate
        diag_oss_licenses.setWindowTitle(_translate("diag_oss_licenses", "Open-Source licenses"))
        self.tb_oss_licenses.setSortingEnabled(True)
        item = self.tb_oss_licenses.horizontalHeaderItem(0)
        item.setText(_translate("diag_oss_licenses", "Software"))
        item = self.tb_oss_licenses.horizontalHeaderItem(1)
        item.setText(_translate("diag_oss_licenses", "License"))
        self.action_start.setText(_translate("diag_oss_licenses", "More licenses..."))
        self.action_start.setToolTip(_translate("diag_oss_licenses", "Show more open-source software licenses"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    diag_oss_licenses = QtWidgets.QDialog()
    ui = Ui_diag_oss_licenses()
    ui.setupUi(diag_oss_licenses)
    diag_oss_licenses.show()
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-
"""Revolution Pi PLC program configuration."""
__author__ = "Sven Sager"
__copyright__ = "Copyright (C) 2023 Sven Sager"
__license__ = "GPLv2"

import gzip
import os
import tarfile
import zipfile
from logging import getLogger
from shutil import rmtree
from tempfile import mkdtemp
from xmlrpc.client import Binary

from PyQt5 import QtCore, QtGui, QtWidgets

from . import helper
from .ui.revpiprogram_ui import Ui_diag_program

log = getLogger(__name__)


class RevPiProgram(QtWidgets.QDialog, Ui_diag_program):
    """Program options of RevPiPyLoad."""

    def __init__(self, parent=None):
        super().__init__(parent)
        self.setupUi(self)
        self.setFixedSize(self.size())

        self.dc = {}
        self.lst_files = []

        self.cbx_pictory.setEnabled(False)
        self.recover_gui()

    def _apply_acl(self):
        """Set availability of controls depending on ACL level."""

        # Setting properties require level 4
        self.cbb_plcprogram.setEnabled(helper.cm.xml_mode >= 4)
        self.txt_plcarguments.setEnabled(helper.cm.xml_mode >= 4)
        self.cbx_plcworkdir_set_uid.setEnabled(helper.cm.xml_mode >= 4)

        # Downloads require level 2
        self.btn_program_download.setEnabled(helper.cm.xml_mode >= 2)
        self.btn_pictory_download.setEnabled(helper.cm.xml_mode >= 2)
        self.btn_procimg_download.setEnabled(helper.cm.xml_mode >= 2)

        # Uploads require level 3
        self.btn_program_upload.setEnabled(helper.cm.xml_mode >= 3)
        self.btn_pictory_upload.setEnabled(helper.cm.xml_mode >= 3)

    def _changesdone(self):
        """
        Check for unsaved changes in dialog.

        :return: True, if unsaved changes was found
        """
        return self.cbb_plcprogram.currentText() != self.dc.get("plcprogram", "") or \
            self.txt_plcarguments.text() != self.dc.get("plcarguments", "") or \
            int(self.cbx_plcworkdir_set_uid.isChecked()) != self.dc.get("plcworkdir_set_uid", 0) or \
            self.sbx_plcprogram_watchdog.value() != self.dc.get("plcprogram_watchdog", 0)

    def _load_settings(self, files_only=False):
        """Load values to GUI widgets."""
        log.debug("RevPiProgram._load_settings")

        if files_only:
            mrk_program = self.cbb_plcprogram.currentText()
        else:
            mrk_program = ""

        self.cbb_plcprogram.clear()
        self.cbb_plcprogram.addItem("")

        self.lst_files.sort()
        is_in_list = False
        for file in self.lst_files:
            if file == ".placeholder":
                continue
            self.cbb_plcprogram.addItem(file)
            check = mrk_program or self.dc.get("plcprogram", "")
            if file == check:
                is_in_list = True

        if not is_in_list:
            log.warning("File {0} is not in list".format(mrk_program or self.dc.get("plcprogram", "")))

        if files_only:
            self.cbb_plcprogram.setCurrentText(mrk_program)
        else:
            self.cbb_plcprogram.setCurrentText(self.dc.get("plcprogram", ""))
            self.txt_plcarguments.setText(self.dc.get("plcarguments", ""))
            self.cbx_plcworkdir_set_uid.setChecked(bool(self.dc.get("plcworkdir_set_uid", 0)))
            self.sbx_plcprogram_watchdog.setValue(self.dc.get("plcprogram_watchdog", 0))

    def accept(self) -> None:
        # todo: After upload ask for restart pcl program?
        if not self._changesdone():
            super().accept()
            return

        if self.cbb_plcprogram.currentText() == "":
            QtWidgets.QMessageBox.critical(
                self, self.tr("Error"), self.tr(
                    "You have to select a start program, before uploading the "
                    "settings."
                )
            )
            return

        ask = QtWidgets.QMessageBox.question(
            self, self.tr("Question"), self.tr(
                "The settings will be set on the Revolution Pi now.\n\n"
                "If you made changes on the 'PCL Program' section, your plc "
                "program will restart now!"
            )
        ) == QtWidgets.QMessageBox.Yes

        if not ask:
            return

        self.dc["plcprogram"] = self.cbb_plcprogram.currentText()
        self.dc["plcarguments"] = self.txt_plcarguments.text()
        self.dc["plcworkdir_set_uid"] = int(self.cbx_plcworkdir_set_uid.isChecked())
        self.dc["plcprogram_watchdog"] = self.sbx_plcprogram_watchdog.value()

        saved = helper.cm.call_remote_function(
            "set_config", self.dc, ask,
            default_value=False
        )

        if saved:
            super().accept()
        else:
            QtWidgets.QMessageBox.critical(
                self, self.tr("Error"), self.tr(
                    "The settings could not be saved on the Revolution Pi!\n"
                    "Try to save the values one mor time and check the log "
                    "files of RevPiPyLoad if the error rises again."
                )
            )

    def closeEvent(self, a0: QtGui.QCloseEvent) -> None:
        if self._changesdone():
            ask = QtWidgets.QMessageBox.question(
                self, self.tr("Question"), self.tr(
                    "Do you really want to quit? \nUnsaved changes will be lost."
                )
            ) == QtWidgets.QMessageBox.Yes

            if ask:
                self.reject()
            else:
                a0.ignore()

    def exec(self) -> int:
        # Reset class values
        if not helper.cm.connected:
            return QtWidgets.QDialog.Rejected

        self.dc = helper.cm.call_remote_function("get_config", default_value={})
        self.lst_files = helper.cm.call_remote_function("get_filelist", default_value=None)
        if len(self.dc) == 0 or self.lst_files is None:
            return QtWidgets.QDialog.Rejected

        self._load_settings()
        self._apply_acl()

        return super().exec()

    def reject(self) -> None:
        """Reject all sub windows and reload settings."""
        self._load_settings()
        super().reject()

    # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
    # region #      REGION: PLC program

    def _upload_pictory(self, filename: str):
        """Upload piCtory configuration."""
        fh = open(filename, "rb")
        file_binary = Binary(fh.read())
        fh.close()

        ask = QtWidgets.QMessageBox.question(
            self, self.tr("Reset driver..."), self.tr(
                "Reset piControl driver after successful uploading new piCtory "
                "configuration?\nThe process image will be interrupted for a "
                "short time!"
            ), QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No | QtWidgets.QMessageBox.Cancel
        )
        if ask == QtWidgets.QMessageBox.Cancel:
            return

        ec = helper.cm.call_remote_function(
            "set_pictoryrsc", file_binary, ask == QtWidgets.QMessageBox.Yes
        )

        if ec is None:
            QtWidgets.QMessageBox.critical(
                self, self.tr("Error"), self.tr(
                    "Got an network error while send data to Revolution Pi.\n"
                    "Please try again."
                )
            )
        elif ec == 0:
            helper.cm.settings.last_pictory_file = filename
            if ask == QtWidgets.QMessageBox.Yes:
                QtWidgets.QMessageBox.information(
                    self, self.tr("Success"), self.tr(
                        "The transfer of the piCtory configuration "
                        "and the reset of piControl have been "
                        "successfully executed."
                    ),
                )
            else:
                QtWidgets.QMessageBox.information(
                    self, self.tr("Success"), self.tr(
                        "The piCtory configuration was successfully transferred."
                    )
                )

        elif ec == -1:
            QtWidgets.QMessageBox.critical(
                self, self.tr("Error"), self.tr(
                    "Can not process the transferred file."
                )
            )
        elif ec == -2:
            QtWidgets.QMessageBox.critical(
                self, self.tr("Error"), self.tr(
                    "Can not find main elements in piCtory file."
                )
            )
        elif ec == -4:
            QtWidgets.QMessageBox.critical(
                self, self.tr("Error"), self.tr(
                    "Contained devices could not be found on Revolution "
                    "Pi. The configuration may be from a newer piCtory version!"
                )
            )
        elif ec == -5:
            QtWidgets.QMessageBox.critical(
                self, self.tr("Error"), self.tr(
                    "Could not load RAP catalog on Revolution Pi."
                )
            )
        elif ec < 0:
            QtWidgets.QMessageBox.critical(
                self, self.tr("Error"), self.tr(
                    "The piCtory configuration could not be "
                    "written on the Revolution Pi."
                )
            )
        elif ec > 0:
            QtWidgets.QMessageBox.warning(
                self, self.tr("Warning"), self.tr(
                    "The piCtroy configuration has been saved successfully.\n"
                    "An error occurred on piControl reset!"
                )
            )

    def check_replacedir(self, rootdir: str):
        """
        Return root dir of a extracted archive an piCtory configuration.

        This function checks the root dir of a extracted directory. It
        will check the existence of a piCtory configuration will will
        return that information in the return tuple.

        :param rootdir: Dir to check
        :return: Tuple with corrected root dir and piCtory configuration
        """
        lst_dir = os.listdir(rootdir)
        if len(lst_dir) == 1 and \
                os.path.isdir(os.path.join(rootdir, lst_dir[0])):
            return os.path.join(rootdir, lst_dir[0]), None

        if len(lst_dir) == 2:
            rscfile = None
            for fname in lst_dir:
                if fname.find(".rsc"):
                    rscfile = os.path.join(rootdir, fname)
            return os.path.join(rootdir, lst_dir[0]), rscfile

        else:
            return rootdir, None

    def create_filelist(self, rootdir):
        """Create a file list of a directory.

        :param rootdir: Root dir to crate file list
        :return: File list with <class 'str'>
        """
        filelist = []
        for tup_dir in os.walk(rootdir):
            for fname in tup_dir[2]:
                filelist.append(os.path.join(tup_dir[0], fname))
        return filelist

    def recover_gui(self):
        """Load saved GUI states."""
        self.cbb_format.setCurrentIndex(helper.settings.value("program/cbb_format_index", 0, int))
        self.cbx_pictory.setChecked(helper.settings.value("program/cbx_pictory_checked", False, bool))
        self.cbx_clear.setChecked(helper.settings.value("program/cbx_clear_checked", False, bool))

    @QtCore.pyqtSlot(int)
    def on_cbb_format_currentIndexChanged(self, index: int):
        helper.settings.setValue("program/cbb_format_index", index)
        self.cbx_pictory.setEnabled(index >= 1)

    @QtCore.pyqtSlot()
    def on_btn_program_download_clicked(self):
        """Download plc program from Revolution Pi."""
        if not helper.cm.connected:
            return

        if self.cbb_format.currentIndex() == 0:
            # Save files as zip archive
            diag_save = QtWidgets.QFileDialog(
                self, self.tr("Save ZIP archive..."),
                helper.cm.settings.last_zip_file or "{0}.zip".format(helper.cm.settings.name),
                self.tr("ZIP archive (*.zip);;All files (*.*)")
            )
            diag_save.setAcceptMode(QtWidgets.QFileDialog.AcceptSave)
            diag_save.setDefaultSuffix("zip")
            self.rejected.connect(diag_save.reject)

            if diag_save.exec() != QtWidgets.QFileDialog.AcceptSave or len(diag_save.selectedFiles()) != 1:
                return
            filename = diag_save.selectedFiles()[0]
            fh = open(filename, "wb")

            helper.cm.settings.last_zip_file = filename

        elif self.cbb_format.currentIndex() == 1:
            # Save files as TarGz archive
            diag_save = QtWidgets.QFileDialog(
                self, self.tr("Save TGZ archive..."),
                helper.cm.settings.last_tar_file or "{0}.tgz".format(helper.cm.settings.name),
                self.tr("TGZ archive (*.tgz);;All files (*.*)")
            )
            diag_save.setAcceptMode(QtWidgets.QFileDialog.AcceptSave)
            diag_save.setDefaultSuffix("tgz")
            self.rejected.connect(diag_save.reject)

            if diag_save.exec() != QtWidgets.QFileDialog.AcceptSave or len(diag_save.selectedFiles()) != 1:
                return
            filename = diag_save.selectedFiles()[0]
            fh = open(filename, "wb")

            helper.cm.settings.last_tar_file = filename

        else:
            # Other indexes are not allowed for download
            return

        plcfile = helper.cm.call_remote_function(
            "plcdownload",
            "zip" if self.cbb_format.currentIndex() == 0 else "tar",
            self.cbx_pictory.isChecked()
        )

        if plcfile is None:
            QtWidgets.QMessageBox.critical(
                self, self.tr("Error"), self.tr(
                    "Could not load PLC program from Revolution Pi."
                )
            )

        else:
            try:
                fh.write(plcfile.data)
                fh.close()

            except Exception as e:
                log.error(e)
                QtWidgets.QMessageBox.critical(
                    self, self.tr("Error"), self.tr(
                        "Coud not save the archive or extract the files!\n"
                        "Please retry.")
                )
            else:
                QtWidgets.QMessageBox.information(
                    self, self.tr("Success"), self.tr(
                        "Transfer successfully completed."
                    )
                )

    @QtCore.pyqtSlot()
    def on_btn_program_upload_clicked(self):
        """Upload plc program to Revolution Pi."""
        if not helper.cm.connected:
            return

        dirtmp = None

        def remove_temp():
            # Remove temp dir
            if dirtmp is not None:
                rmtree(dirtmp)

        dirselect = ""
        lst_files = []
        folder_name = ""
        rscfile = None

        if self.cbb_format.currentIndex() == 0:
            # Upload zip archive content
            diag_open = QtWidgets.QFileDialog(
                self, self.tr("Upload content of ZIP archive..."),
                helper.cm.settings.last_file_upload,
                self.tr("ZIP archive (*.zip);;All files (*.*)")
            )
            diag_open.setAcceptMode(QtWidgets.QFileDialog.AcceptOpen)
            diag_open.setFileMode(QtWidgets.QFileDialog.ExistingFile)
            diag_open.setDefaultSuffix("zip")
            self.rejected.connect(diag_open.reject)

            if diag_open.exec() != QtWidgets.QFileDialog.AcceptSave or len(diag_open.selectedFiles()) != 1:
                return

            filename = diag_open.selectedFiles()[0]
            helper.cm.settings.last_file_upload = filename
            if zipfile.is_zipfile(filename):
                dirtmp = mkdtemp()
                fhz = zipfile.ZipFile(filename)
                fhz.extractall(dirtmp)
                fhz.close()

                lst_files = self.create_filelist(dirtmp)
                dirselect, rscfile = self.check_replacedir(dirtmp)

            else:
                QtWidgets.QMessageBox.critical(
                    self, self.tr("Error"), self.tr(
                        "The selected file ist not a ZIP archive."
                    )
                )
                return

        elif self.cbb_format.currentIndex() == 1:
            # Upload TarGz content
            diag_open = QtWidgets.QFileDialog(
                self, self.tr("Upload content of TAR archive..."),
                helper.cm.settings.last_file_upload,
                self.tr("TAR archive (*.tgz);;All files (*.*)")
            )
            diag_open.setAcceptMode(QtWidgets.QFileDialog.AcceptOpen)
            diag_open.setFileMode(QtWidgets.QFileDialog.ExistingFile)
            diag_open.setDefaultSuffix("tgz")
            self.rejected.connect(diag_open.reject)

            if diag_open.exec() != QtWidgets.QFileDialog.AcceptSave or len(diag_open.selectedFiles()) != 1:
                return

            filename = diag_open.selectedFiles()[0]
            helper.cm.settings.last_file_upload = filename
            if tarfile.is_tarfile(filename):
                dirtmp = mkdtemp()
                fht = tarfile.open(filename)
                fht.extractall(dirtmp)
                fht.close()

                lst_files = self.create_filelist(dirtmp)
                dirselect, rscfile = self.check_replacedir(dirtmp)

            else:
                QtWidgets.QMessageBox.critical(
                    self, self.tr("Error"), self.tr(
                        "The selected file ist not a TAR archive."
                    )
                )
                return

        # No files selected
        if len(lst_files) == 0:
            QtWidgets.QMessageBox.warning(
                self, self.tr("No files to upload..."), self.tr(
                    "Found no files to upload in given location or archive."
                )
            )
            remove_temp()
            return

        # Clean up directory before upload
        clean_revpi = helper.cm.call_remote_function("plcuploadclean", default_value=False)
        if self.cbx_clear.isChecked() and not clean_revpi:
            QtWidgets.QMessageBox.critical(
                self, self.tr("Error"), self.tr(
                    "There was an error deleting the files on the Revolution Pi.\n"
                    "Upload aborted! Please try again."
                )
            )
            remove_temp()
            return

        plc_program_not_in_upload = True
        ec = 0

        for file_name in lst_files:

            if file_name == rscfile:
                # Do not send piCtory configuration
                continue

            # fixme: Fehlerabfang bei Dateilesen
            with open(file_name, "rb") as fh:

                # Generate relative file name for transfer
                if dirselect == "":
                    sendname = os.path.basename(file_name)
                else:
                    # Append folder name on complete folder transfer to crate it on Revolution Pi
                    sendname = os.path.join(
                        folder_name,
                        file_name.replace(dirselect, "")[1:]
                    )

                # Try to find given plc start program in upload files
                if sendname == self.dc.get("plcprogram", ""):
                    plc_program_not_in_upload = False

                ustatus = helper.cm.call_remote_function(
                    "plcupload", Binary(gzip.compress(fh.read())), sendname
                )
                if ustatus is None:
                    ec = -2
                    break
                elif not ustatus:
                    ec = -1
                    break

        if ec == 0:
            QtWidgets.QMessageBox.information(
                self, self.tr("Success"), self.tr(
                    "The PLC program was transferred successfully."
                )
            )
            self.lst_files = helper.cm.call_remote_function("get_filelist", default_value=[])
            self._load_settings(files_only=True)
            if plc_program_not_in_upload:
                QtWidgets.QMessageBox.warning(
                    self, self.tr("Information"), self.tr(
                        "Could not find the selected PLC start program in "
                        "uploaded files.\nThis is not an error, if the file "
                        "was already on the Revolution Pi. Check PLC start "
                        "program field"
                    )
                )

            if self.cbx_pictory.isChecked():
                if rscfile is not None:
                    self._upload_pictory(rscfile)
                else:
                    QtWidgets.QMessageBox.critical(
                        self, self.tr("Error"), self.tr(
                            "There is no piCtory configuration in this archive."
                        )
                    )

        elif ec == -1:
            QtWidgets.QMessageBox.critical(
                self, self.tr("Error"), self.tr(
                    "The Revolution Pi could not process some parts of the transmission."
                )
            )

        elif ec == -2:
            QtWidgets.QMessageBox.critical(
                self, self.tr("Error"), self.tr(
                    "Errors occurred during transmission."
                )
            )

        remove_temp()

    # endregion # # # # #

    # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
    # region #      REGION: Control files

    @QtCore.pyqtSlot()
    def on_btn_pictory_download_clicked(self):
        """Download piCtory configuration."""
        if not helper.cm.connected:
            return

        diag_save = QtWidgets.QFileDialog(
            self, self.tr("Save piCtory file..."),
            os.path.join(
                helper.cm.settings.last_dir_pictory,
                "{0}.rsc".format(helper.cm.settings.name)
            ),
            self.tr("piCtory file (*.rsc);;All files (*.*)")
        )
        diag_save.setAcceptMode(QtWidgets.QFileDialog.AcceptSave)
        diag_save.setDefaultSuffix("rsc")
        self.rejected.connect(diag_save.reject)

        if diag_save.exec() != QtWidgets.QFileDialog.AcceptSave or len(diag_save.selectedFiles()) != 1:
            return

        filename = diag_save.selectedFiles()[0]
        helper.cm.settings.last_dir_pictory = os.path.dirname(filename)
        bin_buffer = helper.cm.call_remote_function("get_pictoryrsc")  # type: Binary
        if bin_buffer is None:
            QtWidgets.QMessageBox.critical(
                self, self.tr("Error"), self.tr(
                    "Could not load piCtory file from Revolution Pi."
                )
            )
        else:
            fh = open(filename, "wb")
            fh.write(bin_buffer.data)
            fh.close()

            QtWidgets.QMessageBox.information(
                self, self.tr("Success"), self.tr(
                    "piCtory configuration successfully loaded and saved to:\n{0}."
                ).format(filename)
            )

    @QtCore.pyqtSlot()
    def on_btn_pictory_upload_clicked(self):
        if not helper.cm.connected:
            return

        diag_open = QtWidgets.QFileDialog(
            self, self.tr("Upload piCtory file..."),
            helper.cm.settings.last_pictory_file or "{0}.rsc".format(helper.cm.settings.name),
            self.tr("piCtory file (*.rsc);;All files (*.*)")
        )
        diag_open.setAcceptMode(QtWidgets.QFileDialog.AcceptOpen)
        diag_open.setFileMode(QtWidgets.QFileDialog.ExistingFile)
        diag_open.setDefaultSuffix("rsc")
        self.rejected.connect(diag_open.reject)

        if diag_open.exec() != QtWidgets.QFileDialog.AcceptSave or len(diag_open.selectedFiles()) != 1:
            return

        self._upload_pictory(diag_open.selectedFiles()[0])

    @QtCore.pyqtSlot()
    def on_btn_procimg_download_clicked(self):
        """Download process image."""
        if not helper.cm.connected:
            return

        diag_save = QtWidgets.QFileDialog(
            self,
            self.tr("Save piControl file..."),
            os.path.join(
                helper.cm.settings.last_dir_picontrol,
                "{0}.img".format(helper.cm.settings.name)
            ),
            self.tr("Process image file (*.img);;All files (*.*)")
        )
        diag_save.setAcceptMode(QtWidgets.QFileDialog.AcceptSave)
        diag_save.setDefaultSuffix("img")
        self.rejected.connect(diag_save.reject)

        if diag_save.exec() != QtWidgets.QFileDialog.AcceptSave or len(diag_save.selectedFiles()) != 1:
            return

        filename = diag_save.selectedFiles()[0]
        helper.cm.settings.last_dir_picontrol = os.path.dirname(filename)
        bin_buffer = helper.cm.call_remote_function("get_procimg")  # type: Binary

        if bin_buffer is None:
            QtWidgets.QMessageBox.critical(
                self, self.tr("Error"), self.tr(
                    "Could not load process image from Revolution Pi."
                )
            )
        else:
            fh = open(filename, "wb")
            fh.write(bin_buffer.data)
            fh.close()

            QtWidgets.QMessageBox.information(
                self, self.tr("Success"), self.tr(
                    "Process image successfully loaded and saved to:\n{0}."
                ).format(filename)
            )

    # endregion # # # # #



# -*- coding: utf-8 -*-
"""Open-Source softwrae licenses."""
__author__ = "Sven Sager"
__copyright__ = "Copyright (C) 2023 Sven Sager"
__license__ = "GPLv2"

from json import load
from logging import getLogger
from os.path import exists
from typing import List

from PyQt5 import QtCore, QtWidgets

from revpicommander.ui.oss_licenses_ui import Ui_diag_oss_licenses

log = getLogger(__name__)


class OssLicenses(QtWidgets.QDialog, Ui_diag_oss_licenses):
    def __init__(self, oss_license_file: str, parent=None):
        super().__init__(parent)
        self.setupUi(self)

        self._lst_licenses: List[dict] = []
        self._oss_license_file = oss_license_file

        self.action_start.setVisible(exists(oss_license_file))

    def _load_license_file(self) -> None:
        if exists(self._oss_license_file):
            try:
                with open(self._oss_license_file) as fh:
                    self._lst_licenses = load(fh)
            except Exception as e:
                log.error("Could not load oss license file: '{0}'".format(e))

        for i in range(len(self._lst_licenses)):
            dict_license = self._lst_licenses[i]
            tb_item_name = QtWidgets.QTableWidgetItem(dict_license.get("Name", ""))
            tb_item_name.setData(QtCore.Qt.UserRole, i)
            tb_item_license = QtWidgets.QTableWidgetItem(dict_license.get("License", ""))
            tb_item_license.setToolTip(tb_item_license.text())
            tb_item_license.setData(QtCore.Qt.UserRole, i)

            self.tb_oss_licenses.insertRow(i)
            self.tb_oss_licenses.setItem(i, 0, tb_item_name)
            self.tb_oss_licenses.setItem(i, 1, tb_item_license)

        self.tb_oss_licenses.resizeColumnsToContents()

    def exec(self) -> int:
        # Prevent loading every time the program is starting
        if not self._lst_licenses:
            self._load_license_file()

        return super().exec()

    @QtCore.pyqtSlot(QtWidgets.QTableWidgetItem, QtWidgets.QTableWidgetItem)
    def on_tb_oss_licenses_currentItemChanged(
        self,
        current: QtWidgets.QTableWidgetItem,
        previous: QtWidgets.QTableWidgetItem,
    ):
        log.debug("Enter slot on_tb_oss_licenses_currentItemChanged")
        license_index = current.data(QtCore.Qt.UserRole)
        license_object = self._lst_licenses[license_index]
        license_object["LicenseText"] = license_object["LicenseText"].replace("\n", "<br>")
        self.txt_license.setHtml(
            """<h2>{Name}</h2>
<p>{Description}</p>
<p>
    <ul>
        <li>Version: {Version}</li>
        <li>Author: {Author}</li>
        <li>URL: <a href="{URL}">{URL}</a></li>
    </ul>
</p>
<h3>License: {License}</h3>
<p>
    <code>{LicenseText}</code>
</p>""".format(
                **license_object
            )
        )



# -*- coding: utf-8 -*-
"""Start main application of this package."""
__author__ = "Sven Sager"
__copyright__ = "Copyright (C) 2023 Sven Sager"
__license__ = "GPLv2"

# If we are running from a wheel, add the wheel to sys.path
if __package__ == "":
    from os.path import dirname
    from sys import path

    # __file__ is package-*.whl/package/__main__.py
    # Resulting path is the name of the wheel itself
    package_path = dirname(dirname(__file__))
    path.insert(0, package_path)

if __name__ == "__main__":
    import sys

    try:
        # Use absolut import in the __main__ module
        from revpicommander.revpicommander import main

        # Run the main application of this package
        sys.exit(main())

    except Exception as e:
        sys.stdout.write(f"Can not start __main__ module: {e}")
        sys.stdout.write("\n")
        sys.exit(1)
